/*********************************************************/
/* modlisation d'un moteur  explosion                  */
/* CAISSON                                               */
/* ----------------------------------------------------- */
/* Georges CAMY / Arnaud QUENUM                          */
/* IMAC Ing3 2002/2003 - Universit de Marne La Valle  */
/* Copyright 2003                                        */
/*********************************************************/
#include "Caisson.h"


/*********************************************************/
/*                    constructeur                       */
/*********************************************************/
Caisson::Caisson(){

  // TexturedCube
  mon_cube = new TexturedCube();

  // dfinitions: matriaux
  float mat_ambi_ag[]= {0.2,0.2,0.2},mat_diff_ag[]= {0.3,0.3,0.3},
	mat_spec_ag[]= {0.0,0.0,0.0},mat_nphong_ag=10.0,
	mat_ambi_or[]= {0.2,0.2,0.1},mat_diff_or[]= {0.3,0.3,0.2},
        mat_spec_or[]= {0.8,0.8,0.1},mat_nphong_or=10.0;


  // cration de l'objet caisson
  id_caisson = glGenLists(1);
  glNewList(id_caisson,GL_COMPILE);

  // paramtres du modle d'illumination
  glMaterialfv(GL_FRONT,GL_AMBIENT,mat_ambi_ag);
  glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diff_ag);
  glMaterialfv(GL_FRONT,GL_SPECULAR,mat_spec_ag);
  glMaterialf(GL_FRONT,GL_SHININESS,mat_nphong_ag);

  // bordure et sol
  glPushMatrix();
  glTranslatef(-3.2, 0.0,-0.8);
  glRotatef(36,0.0,0.0,1);
  DrawTools::drawCylinder(3.2,1.8,35,1,false,1.6*M_PI);
  glTranslatef(-0.0, 0.0,-1.2);
  DrawTools::drawCylinder(3.2,1.4,35,1,true);
  glPopMatrix();

  // partie infrieur sol
  glPushMatrix();
  glTranslatef(-3.2, 0.0,-2.0);
  glRotatef(180,0,1,0);
  glutSolidCone(3.2, 1.8, 20, 1);
  DrawTools::drawCylinder(1.7,1.8,35,1,true);
  glPopMatrix();

  // coque du piston
  glPushMatrix();
  glTranslatef(8.0,0.0,0.0);
  glRotatef(-90,1,0,0);
  glRotatef(-90,0,1,0);
  glTranslatef(0.0,0.0,0.2);
  DrawTools::drawInvertedCylinder(2,8.5,35,1,false,M_PI);
  DrawTools::drawCylinder(3,10.0,35,1,false,M_PI);
  glPopMatrix();

  // fermeture suprieure gauche de la coque piston
  glPushMatrix();
  glTranslatef(3.6,2.5,0.5);
  glScalef(8.7,1,1);
  mon_cube->dessiner(1);
  glPopMatrix();

 // fermeture suprieure droite de la coque piston
  glPushMatrix();
  glTranslatef(3.6,-2.5,0.5);
  glScalef(8.7,1,1);
  mon_cube->dessiner(1);
  glPopMatrix();

  // joint suprieur gauche coque piston - sol
  glPushMatrix();
  glRotatef(50,0,0,1);
  glTranslatef(1.6,2.5,-0.4);
  glScalef(1,2.5,2.35);
  mon_cube->dessiner(1.2);
  glPopMatrix();
  glPushMatrix();
  glTranslatef(-2.4,3.7,-0.4);
  glScalef(1.3,1,2.35);
  mon_cube->dessiner(1.2);
  glPopMatrix();

  // joint suprieur droite coque piston - sol
  glPushMatrix();
  glRotatef(-50,0,0,1);
  glTranslatef(1.6,-2.5,-0.4);
  glScalef(1,2.5,2.35);
  mon_cube->dessiner(1.2);
  glPopMatrix();
  glPushMatrix();
  glTranslatef(-2.4,-3.7,-0.4);
  glScalef(1.3,1,2.35);
  mon_cube->dessiner(1.2);
  glPopMatrix();

  // tte caisson partie basse
  glPushMatrix();
  glTranslatef(10.2,0,-1);
  glScalef(1,10,3.9);
  mon_cube->dessiner(1);
  glPopMatrix();

  // tte caisson partie ct droit
  glPushMatrix();
  glTranslatef(7.8,3.25,-1);
  glScalef(0.6,3.5,3.9);
  mon_cube->dessiner(1);
  glPopMatrix();

  // tte caisson partie ct gauche
  glPushMatrix();
  glTranslatef(7.8,-3.25,-1);
  glScalef(0.6,3.5,3.9);
  mon_cube->dessiner(1);
  glPopMatrix();

  // tte caisson partie suprieur
  glPushMatrix();
  glTranslatef(9,0,-1.75);
  glScalef(3,10,2.5);
  mon_cube->dessiner(1);
  glPopMatrix();

  // tte caisson diviseur central
  glPushMatrix();
  glTranslatef(9,0,0.5);
  glScalef(3,1,1);
  mon_cube->dessiner(1);
  glPopMatrix();

  // fin de dclaration de l'objet caisson
  glEndList();


  // cration de l'objet bougie
  id_bougie = glGenLists(1);
  glNewList(id_bougie,GL_COMPILE);

  // paramtres du modle d'illumination
  glMaterialfv(GL_FRONT,GL_AMBIENT,mat_ambi_or);
  glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diff_or);
  glMaterialfv(GL_FRONT,GL_SPECULAR,mat_spec_or);
  glMaterialf(GL_FRONT,GL_SHININESS,mat_nphong_or);

  glPushMatrix();
  glTranslatef(7.6,0,0);
  glRotatef(-90,0,1,0);
  glutSolidCone(0.2, 0.4, 20, 1);
  glPopMatrix();

  // fin de dclaration de l'objet caisson
  glEndList();
}



/*********************************************************/
/*                    dessiner                           */
/*********************************************************/
void Caisson::dessiner(bool texture){

  if (texture){
    // charger la texture
    image=loadPPM("textureAluminum.ppm",&dx,&dy);
    // demande d'un identificateur de texture libre
    glGenTextures(1,&id_texture);
    // choix de la texture courrante
    glBindTexture(GL_TEXTURE_2D,id_texture);
    // organisation des plans dans le tableau
    glPixelStorei(GL_UNPACK_ALIGNMENT,1);
    // paramtre de rptition de la texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
    // la texture sera multiplie par la couleur du modle d'illumination
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    // construction d'une texture multichelle  partir d'une image
    gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,dx,dy,GL_RGB,GL_UNSIGNED_BYTE,image);

    // dessiner le caisson
    glEnable(GL_TEXTURE_2D);
    glCallList(id_caisson);
    glDisable(GL_TEXTURE_2D);
  }
  else{
    // dessiner le caisson
    glCallList(id_caisson);
  }

  // dessiner la bougie
  glCallList(id_bougie);

}



/*********************************************************/
/*                    loadPPM                            */
/*********************************************************/
unsigned char * Caisson::loadPPM(char *titre,int *dx,int *dy){
  FILE *fichier;
  char chaine[255];
  unsigned char *image=NULL;

  if (!(fichier = fopen(titre, "rt"))){ 
    printf("fichier %s introuvable\n",titre);
    exit(0);
    return NULL;
  }
  fscanf(fichier,"%s\n",chaine);
  //printf("%s\n",chaine);
  do{ 
    fgets(chaine,255,fichier);
  } while (chaine[0]=='#');
  sscanf(chaine,"%d %d",dx,dy);
  //printf("%d,%d\n",*dx,*dy);
  fscanf(fichier,"%s\n",chaine);
  image = (unsigned char *) malloc (*dx**dy*3*sizeof(unsigned char));
  if (image==NULL){ 
    printf("erreur d'allocation de l'image de texture\n");
    return NULL;
  }
  fread (image, sizeof (unsigned char), *dx**dy * 3, fichier);
  fclose(fichier);

  return image;
}

