/*********************************************************/
/* modlisation d'un moteur  explosion                  */
/* PISTON                                                */
/* ----------------------------------------------------- */
/* Georges CAMY / Arnaud QUENUM                          */
/* IMAC Ing3 2002/2003 - Universit de Marne La Valle  */
/* Copyright 2003                                        */
/*********************************************************/
#include "Piston.h"



/*********************************************************/
/*                    constructeur                       */
/*********************************************************/
Piston::Piston(){

  // dimensions
  d = 1.5;
  l = 5.0;
  e = 0.1;
  
  // initialisation angles
  alpha_r = 0;
  beta_r = 0;
  beta_d = 0;

  // dfinitions: matriaux
  float mat_ambi_or[]= {0.2,0.2,0.1},mat_diff_or[]= {0.8,0.8,0.2},
        mat_spec_or[]= {0.8,0.8,0.8},mat_nphong_or=20.0,
	mat_ambi_ag[]= {0.2,0.2,0.2},mat_diff_ag[]= {0.8,0.8,0.8},
	mat_spec_ag[]= {0.8,0.8,0.8},mat_nphong_ag=40.0,
	mat_ambi_pl[]= {0.2,0.1,0.0},mat_diff_pl[]= {0.9,0.1,0.1},
	mat_spec_pl[]= {0.1,0.1,0.1},mat_nphong_pl=200.0;


  /*******************************************************/
  /*             cration de objet bras1 et bras2        */
  /*******************************************************/
  
		//*****************
		// objet bras1
		//*****************

  id_bras1 = glGenLists(1);
  glNewList(id_bras1,GL_COMPILE);

  // paramtres du modle d'illumination
  glMaterialfv(GL_FRONT,GL_AMBIENT,mat_ambi_or);
  glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diff_or);
  glMaterialfv(GL_FRONT,GL_SPECULAR,mat_spec_or);
  glMaterialf(GL_FRONT,GL_SHININESS,mat_nphong_or);

  // premier petit cylindre central
  glPushMatrix();
  glTranslatef(0, 0.0, e/2.0);
  DrawTools::drawCylinder(0.1,0.2,8,1,true);
  glPopMatrix();


  // deuxieme petit cylindre
  glPushMatrix();
  glTranslatef(d, 0.0, e/2.0);
  DrawTools::drawCylinder(0.1,0.2,8,1,true);
  glPopMatrix();

  // troisieme cylindre, celui du bout arrondi
  glPushMatrix();
  glTranslatef(d,0.0,-0.15);
  glRotatef(-90.0,0.0,0.0,1);
  DrawTools::drawCylinder(0.5,0.3,10,1,true,M_PI);
  glPopMatrix();

  // pav central
  glPushMatrix();
  glScalef(3, 1, 0.3);
  glutSolidCube(1);
  glPopMatrix();

  // quart de fromage bout du bras
  glPushMatrix();
  glRotatef(135.0,0,0,1);
  glTranslatef(0.0,0.0,-0.15);
  DrawTools::drawCylinder(3,0.3,10,1,true,0.5*M_PI);
  glPopMatrix();

  // fermeture du ct gauche
  glPushMatrix();
  glRotatef(45,0,0,1);
  glTranslatef(-1.48,0,0);
  glScalef(3,0.01,0.3);
  glutSolidCube(1);
  glPopMatrix();

  // fermeture du ct droit
  glPushMatrix();
  glRotatef(-45,0,0,1);
  glTranslatef(-1.48,0,0);
  glScalef(3,0.01,0.3);
  glutSolidCube(1);
  glPopMatrix();

  // fin de dclaration de l'objet bras
  glEndList();


		//*****************
		//   objet bras2
		//*****************

  id_bras2 = glGenLists(1);
  glNewList(id_bras2,GL_COMPILE);

  // paramtres du modle d'illumination
  glMaterialfv(GL_FRONT,GL_AMBIENT,mat_ambi_or);
  glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diff_or);
  glMaterialfv(GL_FRONT,GL_SPECULAR,mat_spec_or);
  glMaterialf(GL_FRONT,GL_SHININESS,mat_nphong_or);
  
  glPushMatrix();
  glRotatef(180.0,1.0,0.0,0.0);
  glCallList(id_bras1);
  glPopMatrix();
 glEndList();


  /*******************************************************/
  /*               cration de l'objet arbre             */
  /*******************************************************/
  id_arbre = glGenLists(1);
  glNewList(id_arbre,GL_COMPILE);

  // paramtres du modle d'illumination
  glMaterialfv(GL_FRONT,GL_AMBIENT,mat_ambi_ag);
  glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diff_ag);
  glMaterialfv(GL_FRONT,GL_SPECULAR,mat_spec_ag);
  glMaterialf(GL_FRONT,GL_SHININESS,mat_nphong_ag);

  // attache 1
  glutSolidCube(7*e);

  // attache 2
  glPushMatrix();
  glTranslatef(l,0.0,0.0);
  glutSolidCube(7*e);
  glPopMatrix();

  // bras
  glPushMatrix();
  glTranslatef(e/2.0,0.0,0.0);
  glRotatef(90.0,0.0,1.0,0.0);
  DrawTools::drawCylinder(3*e,l-e,10,10,false);
  glPopMatrix();

  // fin de dclaration de l'objet arbre
  glEndList();


  /*******************************************************/
  /*               cration de l'objet piston            */
  /*******************************************************/
  id_piston = glGenLists(1);
  glNewList(id_piston,GL_COMPILE);

  // paramtres du modle d'illumination
  glMaterialfv(GL_FRONT,GL_AMBIENT,mat_ambi_pl);
  glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diff_pl);
  glMaterialfv(GL_FRONT,GL_SPECULAR,mat_spec_pl);
  glMaterialf(GL_FRONT,GL_SHININESS,mat_nphong_pl);

  // piston
  glPushMatrix();

  glRotatef(90.0,0.0,1.0,0.0);
  glTranslatef(-e, 0.0, 0.0);
  DrawTools::drawCylinder(2.,2,35,1,true,2*M_PI);

  glTranslatef(0.0, 0.0, 2.2);
  DrawTools::drawCylinder(2.,0.20,35,1,true,2*M_PI);

  glTranslatef(0.0, 0.0,0.4);
  DrawTools::drawCylinder(2.,0.20,35,1,true,2*M_PI);

  glTranslatef(0.0, 0.0,0.4);
  DrawTools::drawCylinder(2.,0.20,35,1,true,2*M_PI);

  glPopMatrix();

  
  // fin de dclaration de l'objet piston
  glEndList();
}



/*********************************************************/
/*                    dessiner                           */
/*********************************************************/
void Piston::dessiner(float alpha_d){

   //Angles
   alpha_r = (double)((alpha_d*M_PI)/180);
   beta_r = (double)(asin(d*sin(alpha_r)/l));
   beta_d = (double)(180*beta_r)/M_PI;
   xpiston = d*cos(alpha_r) + l*cos(beta_r) - ((l+d)/2);

   glPushMatrix();
   glTranslatef(-(l+d)/2,0.0,e-0.2);
   glRotatef(alpha_d, 0.0, 0.0, 1.0);
   glCallList(id_bras2);
   glTranslatef(0.0,0.0,0.8);
   glCallList(id_bras1);
   glPopMatrix();
   
   glPushMatrix();
   glTranslatef(xpiston,0,3*e/2);
   glCallList(id_piston);
   glPopMatrix();
   
   glPushMatrix();
   glTranslatef(xpiston,0.0,5*e/2);
   glRotatef(180 - beta_d, 0.0, 0.0, 1.0);
   glCallList(id_arbre);
   glPopMatrix();
}

